@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 5.2, a parsing and reporting tool for gcov.
+# This file is part of gcovr 6.0, a parsing and reporting tool for gcov.
 # https://gcovr.com/en/stable
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2022 the gcovr authors
+# Copyright (c) 2013-2023 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -27,6 +27,9 @@ import re
 import sys
 
 
+GCOVR_ISOLATED_TEST = os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"
+
+
 # The CaptureObject class holds the capture method result
 class CaptureObject:
     def __init__(self, out, err, exception):
@@ -150,6 +153,10 @@ def test_non_existing_directory_html_details(capsys):
     helper_test_non_existing_directory_output(capsys, "--html-details")
 
 
+def test_non_existing_directory_html_nested(capsys):
+    helper_test_non_existing_directory_output(capsys, "--html-nested")
+
+
 def test_non_existing_directory_sonarqube(capsys):
     helper_test_non_existing_directory_output(capsys, "--sonarqube")
 
@@ -189,6 +196,10 @@ def test_non_existing_directory_2_html_details(capsys):
     helper_test_non_existing_directory_2_output(capsys, "--html-details")
 
 
+def test_non_existing_directory_2_html_nested(capsys):
+    helper_test_non_existing_directory_2_output(capsys, "--html-nested")
+
+
 def test_non_existing_directory_2_sonarqube(capsys):
     helper_test_non_existing_directory_2_output(capsys, "--sonarqube")
 
@@ -208,58 +219,47 @@ def helper_test_non_writable_directory_output(capsys, option):  # pragma: no cov
     assert c.exception.code != 0
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_output(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--output")
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_txt(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--txt")
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_xml(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--xml")
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_html(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--html")
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_html_details(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--html-details")
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
+def test_non_writable_directory_html_nested(capsys):  # pragma: no cover
+    helper_test_non_writable_directory_output(capsys, "--html-nested")
+
+
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_sonarqube(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--sonarqube")
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_json(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--json")
 
 
-@pytest.mark.skipif(
-    not (os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"), reason="Only for docker"
-)
+@pytest.mark.skipif(not GCOVR_ISOLATED_TEST, reason="Only for docker")
 def test_non_writable_directory_csv(capsys):  # pragma: no cover
     helper_test_non_writable_directory_output(capsys, "--csv")
 
@@ -275,6 +275,17 @@ def test_no_output_html_details(caplog):
     assert c.exception.code != 0
 
 
+def test_no_output_html_nested(caplog):
+    c = log_capture(caplog, ["--html-nested"])
+    message = c.record_tuples[0]
+    assert message[1] == logging.ERROR
+    assert (
+        message[2]
+        == "a named output must be given, if the option --html-nested\nis used."
+    )
+    assert c.exception.code != 0
+
+
 def test_branch_threshold_nan(capsys):
     c = capture(capsys, ["--fail-under-branch", "nan"])
     assert c.out == ""
@@ -461,7 +472,7 @@ def test_html_injection_via_json(capsys, tmp_path):
 
     script = '<script>alert("pwned")</script>'
     jsondata = {
-        "gcovr/format_version": "0.3",
+        "gcovr/format_version": "0.5",
         "files": [
             {"file": script, "functions": [], "lines": []},
             {"file": "other", "functions": [], "lines": []},
