@@ -2,12 +2,12 @@
 
 #  ************************** Copyrights and license ***************************
 #
-# This file is part of gcovr 5.2, a parsing and reporting tool for gcov.
+# This file is part of gcovr 6.0, a parsing and reporting tool for gcov.
 # https://gcovr.com/en/stable
 #
 # _____________________________________________________________________________
 #
-# Copyright (c) 2013-2022 the gcovr authors
+# Copyright (c) 2013-2023 the gcovr authors
 # Copyright (c) 2013 Sandia Corporation.
 # Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 # the U.S. Government retains certain rights in this software.
@@ -18,7 +18,6 @@
 # ****************************************************************************
 
 import glob
-import io
 import logging
 import os
 import platform
@@ -31,9 +30,12 @@ import difflib
 import zipfile
 
 from yaxmldiff import compare_xml
+from lxml import etree
 
-python_interpreter = sys.executable.replace(
-    "\\", "/"
+from gcovr.utils import force_unix_separator
+
+python_interpreter = force_unix_separator(
+    sys.executable
 )  # use forward slash on windows as well
 env = os.environ
 env["GCOVR"] = python_interpreter + " -m gcovr"
@@ -55,6 +57,8 @@ basedir = os.path.split(os.path.abspath(__file__))[0]
 
 skip_clean = None
 
+GCOVR_ISOLATED_TEST = os.getenv("GCOVR_ISOLATED_TEST") == "zkQEVaBpXF1i"
+
 CC = os.path.split(env["CC"])[1]
 IS_CLANG = True if CC.startswith("clang") else False
 
@@ -77,7 +81,7 @@ REFERENCE_DIRS = []
 REFERENCE_DIR_VERSION_LIST = (
     ["gcc-5", "gcc-6", "gcc-8", "gcc-9", "gcc-10", "gcc-11"]
     if "gcc" in CC_REFERENCE
-    else ["clang-10", "clang-13"]
+    else ["clang-10", "clang-13", "clang-14"]
 )
 for ref in REFERENCE_DIR_VERSION_LIST:
     REFERENCE_DIRS.append(os.path.join("reference", ref))
@@ -106,7 +110,7 @@ RE_COVERALLS_GIT_PRETTY = re.compile(
 
 RE_HTML_ATTRS = re.compile('((timestamp)|(version))="[^"]*"')
 RE_HTML_FOOTER_VERSION = re.compile(
-    r"(Generated by: <a [^>]+>GCOVR \(Version) \d+\.\d+(\)</a>)"
+    r"(Generated by: <a href=\".+/)(?:master|\d+\.\d+)(\">GCOVR \(Version )(?:master|\d+\.\d+)(\)</a>)"
 )
 RE_HTML_HEADER_DATE = re.compile(r"(<td)>\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d<(/td>)")
 
@@ -116,10 +120,7 @@ def scrub_txt(contents):
 
 
 def scrub_csv(contents):
-    contents = contents.replace("\r", "")
-    contents = contents.replace("\n\n", "\n")
-    # Replace windows file separator for html reports generated in Windows
-    contents = contents.replace("\\", "/")
+    contents = force_unix_separator(contents)
     return contents
 
 
@@ -127,22 +128,18 @@ def scrub_xml(contents):
     contents = RE_DECIMAL.sub(lambda m: str(round(float(m.group(1)), 5)), contents)
     contents = RE_XML_ATTRS.sub(r'\1=""', contents)
     contents = RE_XML_GCOVR_VERSION.sub('version=""', contents)
-    contents = contents.replace("\r", "")
     return contents
 
 
 def scrub_html(contents):
     contents = RE_HTML_ATTRS.sub('\\1=""', contents)
-    contents = RE_HTML_FOOTER_VERSION.sub("\\1 4.x\\2", contents)
+    contents = RE_HTML_FOOTER_VERSION.sub("\\1x.x\\2x.x\\3", contents)
     contents = RE_HTML_HEADER_DATE.sub("\\1>0000-00-00 00:00:00<\\2", contents)
-    contents = contents.replace("\r", "")
-    # Replace windows file separator for html reports generated in Windows
-    contents = contents.replace("\\", "/")
+    contents = force_unix_separator(contents)
     return contents
 
 
 def scrub_coveralls(contents):
-    contents += "\n"
     contents = RE_COVERALLS_CLEAN_KEYS.sub('"\\1": ""', contents)
     contents = RE_COVERALLS_GIT_PRETTY.sub("", contents)
     contents = RE_COVERALLS_GIT.sub("", contents)
@@ -153,19 +150,46 @@ def findtests(basedir):
     for f in sorted(os.listdir(basedir)):
         if not os.path.isdir(os.path.join(basedir, f)):
             continue
-        if f.startswith("."):
-            continue
-        if "pycache" in f:
+        if not os.path.isfile(os.path.join(basedir, f, "Makefile")):  # pragma: no cover
             continue
         yield f
 
 
-def assert_xml_equals(reference, coverage):
-    diff = compare_xml(reference, coverage)
-    if diff is None:
-        return
+def assert_equals(reference_file, reference, test_file, test, encoding):
+    _, extension = os.path.splitext(reference_file)
+    if extension in [".html", ".xml"]:
+        if extension == ".html":
+            reference = etree.fromstring(
+                reference.encode(), etree.HTMLParser(encoding=encoding)
+            )
+            test = etree.fromstring(test.encode(), etree.HTMLParser(encoding=encoding))
+        else:
+            reference = etree.fromstring(reference.encode())
+            test = etree.fromstring(test.encode())
 
-    raise AssertionError(f"XML documents differed (-reference, +actual):\n{diff}")
+        diff_out = compare_xml(reference, test)
+        if diff_out is None:
+            return
+
+        diff_out = (
+            f"-- {reference_file}\n++ {test_file}\n{diff_out}"  # pragma: no cover
+        )
+    else:
+        diff_out = list(
+            difflib.unified_diff(
+                reference.splitlines(keepends=True),
+                test.splitlines(keepends=True),
+                fromfile=reference_file,
+                tofile=test_file,
+            )
+        )
+
+        diff_is_empty = len(diff_out) == 0
+        if diff_is_empty:
+            return
+        diff_out = "".join(diff_out)  # pragma: no cover
+
+    raise AssertionError(diff_out)  # pragma: no cover
 
 
 def run(cmd, cwd=None):
@@ -180,11 +204,11 @@ def find_reference_files(output_pattern):
     for reference_dir in REFERENCE_DIRS:
         for pattern in output_pattern:
             if os.path.isdir(reference_dir):
-                for file in glob.glob(os.path.join(reference_dir, pattern)):
-                    if os.path.basename(file) not in seen_files:
-                        coverage = os.path.basename(file)
-                        seen_files.add(coverage)
-                        yield coverage, file
+                for reference_file in glob.glob(os.path.join(reference_dir, pattern)):
+                    if os.path.basename(reference_file) not in seen_files:
+                        test_file = os.path.basename(reference_file)
+                        seen_files.add(test_file)
+                        yield test_file, reference_file
 
 
 @pytest.fixture(scope="module")
@@ -198,14 +222,16 @@ def compiled(request, name):
 
 
 KNOWN_FORMATS = [
+    # Own formats
     "txt",
-    "xml",
     "html",
-    "sonarqube",
     "json",
     "json_summary",
     "csv",
+    # Other formats
+    "cobertura",
     "coveralls",
+    "sonarqube",
 ]
 
 
@@ -269,7 +295,11 @@ def pytest_generate_tests(metafunc):
             marks = [
                 pytest.mark.skipif(
                     name == "simple1-drive-subst" and not IS_WINDOWS,
-                    reason="drive substitution only available on windows",
+                    reason="drive substitution only available on Windows",
+                ),
+                pytest.mark.skipif(
+                    name == "cmake_gtest" and not GCOVR_ISOLATED_TEST,
+                    reason="only available in docker",
                 ),
                 pytest.mark.xfail(
                     name == "exclude-throw-branches"
@@ -294,6 +324,22 @@ def pytest_generate_tests(metafunc):
                     and IS_MACOS,
                     reason="On MacOS the constructor is called twice",
                 ),
+                pytest.mark.xfail(
+                    name == "noncode" and IS_MACOS,
+                    reason="On MacOS the there are other branches",
+                ),
+                pytest.mark.xfail(
+                    name in ["decisions-neg-delta"] and IS_MACOS,
+                    reason="On MacOS there is no branch for std::vector",
+                ),
+                pytest.mark.xfail(
+                    name in ["excl-line-branch"] and IS_MACOS,
+                    reason="On MacOS there are different number of branches generated",
+                ),
+                pytest.mark.xfail(
+                    name in ["wrong-casing"] and not IS_WINDOWS,
+                    reason="Only windows has a case insensitive file system",
+                ),
                 pytest.mark.xfail(
                     name == "gcc-abspath"
                     and (
@@ -349,14 +395,15 @@ def generate_reference_data(output_pattern):  # pragma: no cover
                 shutil.copyfile(generated_file, reference_file)
 
 
-def update_reference_data(coverage_file, reference_file):  # pragma: no cover
+def update_reference_data(reference_file, content, encoding):  # pragma: no cover
     if CC_REFERENCE in reference_file:
         reference_dir = os.path.dirname(reference_file)
     else:
         reference_dir = os.path.join("reference", CC_REFERENCE)
         os.makedirs(reference_dir, exist_ok=True)
         reference_file = os.path.join(reference_dir, os.path.basename(reference_file))
-    shutil.copyfile(coverage_file, reference_file)
+    with open(reference_file, "w", encoding=encoding) as out:
+        out.write(content)
 
     return reference_file
 
@@ -375,7 +422,6 @@ def archive_difference_data(name, coverage_file, reference_file):  # pragma: no
 def remove_duplicate_data(
     encoding, scrub, coverage, coverage_file, reference_file
 ):  # pragma: no cover
-    reference_dir = os.path.dirname(reference_file)
     # Loop over the other coverage data
     for reference_dir in REFERENCE_DIRS:  # pragma: no cover
         other_reference_file = os.path.join(reference_dir, coverage_file)
@@ -383,36 +429,44 @@ def remove_duplicate_data(
         if other_reference_file != reference_file and os.path.isfile(
             other_reference_file
         ):  # pragma: no cover
-            with io.open(other_reference_file, encoding=encoding) as f:
+            with open(other_reference_file, encoding=encoding) as f:
                 if coverage == scrub(f.read()):
                     os.unlink(reference_file)
             break
+        # Check if folder is empty
+        if (
+            os.path.exists(reference_dir)
+            and len(glob.glob(os.path.join(reference_dir, "*"))) == 0
+        ):
+            os.rmdir(reference_dir)
 
 
 SCRUBBERS = dict(
+    # Own formats
     txt=scrub_txt,
-    xml=scrub_xml,
     html=scrub_html,
-    sonarqube=scrub_xml,
     json=lambda x: x,
     json_summary=lambda x: x,
     csv=scrub_csv,
+    # Other formats
+    cobertura=scrub_xml,
     coveralls=scrub_coveralls,
+    sonarqube=scrub_xml,
 )
 
 OUTPUT_PATTERN = dict(
+    # Own formats
     txt=["coverage*.txt"],
-    xml=["coverage*.xml"],
     html=["coverage*.html", "coverage*.css"],
-    sonarqube=["sonarqube*.xml"],
     json=["coverage*.json"],
     json_summary=["summary_coverage*.json"],
     csv=["coverage*.csv"],
+    # Other formats
+    cobertura=["cobertura*.xml"],
     coveralls=["coveralls*.json"],
+    sonarqube=["sonarqube*.xml"],
 )
 
-ASSERT_EQUALS = dict(xml=assert_xml_equals, sonarqube=assert_xml_equals)
-
 
 def test_build(
     compiled,
@@ -425,7 +479,6 @@ def test_build(
     name = compiled
     scrub = SCRUBBERS[format]
     output_pattern = OUTPUT_PATTERN[format]
-    assert_equals = ASSERT_EQUALS.get(format, None)
 
     encoding = "utf8"
     if format == "html" and name.startswith("html-encoding-"):
@@ -438,36 +491,39 @@ def test_build(
         generate_reference_data(output_pattern)
 
     whole_diff_output = []
-    for coverage_file, reference_file in find_reference_files(output_pattern):
-        with io.open(coverage_file, encoding=encoding) as f:
-            coverage = scrub(f.read())
-        with io.open(reference_file, encoding=encoding) as f:
-            reference = scrub(f.read())
+    for test_file, reference_file in find_reference_files(output_pattern):
+        with open(test_file, encoding=encoding) as f:
+            test_scrubbed = scrub(f.read())
+
+        # Overwrite the file created above with the scrubbed content
+        if generate_reference:  # pragma: no cover
+            with open(reference_file, "w", encoding=encoding) as f:
+                f.write(test_scrubbed)
+            reference_scrubbed = test_scrubbed
+        else:
+            with open(reference_file, encoding=encoding) as f:
+                reference_scrubbed = scrub(f.read())
 
         try:
-            if assert_equals is not None:
-                assert_equals(reference, coverage)
-            else:
-                diff_out = list(
-                    difflib.unified_diff(
-                        reference.splitlines(keepends=True),
-                        coverage.splitlines(keepends=True),
-                        fromfile=reference_file,
-                        tofile=coverage_file,
-                    )
-                )
-                diff_is_empty = len(diff_out) == 0
-                assert diff_is_empty, "".join(diff_out)
-        except Exception as e:  # pragma: no cover
-            whole_diff_output += "  " + str(e) + "\n"
+            assert_equals(
+                reference_file,
+                reference_scrubbed,
+                test_file,
+                test_scrubbed,
+                encoding,
+            )
+        except AssertionError as e:  # pragma: no cover
+            whole_diff_output += str(e) + "\n"
             if update_reference:
-                reference_file = update_reference_data(coverage_file, reference_file)
+                reference_file = update_reference_data(
+                    reference_file, test_scrubbed, encoding
+                )
             if archive_differences:
-                archive_difference_data(name, coverage_file, reference_file)
+                archive_difference_data(name, test_file, reference_file)
 
         if generate_reference or update_reference:  # pragma: no cover
             remove_duplicate_data(
-                encoding, scrub, coverage, coverage_file, reference_file
+                encoding, scrub, test_scrubbed, test_file, reference_file
             )
 
     diff_is_empty = len(whole_diff_output) == 0
